"use strict";

var _classCallCheck = require("babel-runtime/helpers/class-call-check")["default"];

var _inherits = require("babel-runtime/helpers/inherits")["default"];

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Visitor = _interopRequire(require("./visitor"));

var getKeyNormalized = require("./translate-util").getKeyNormalized;

/**
 * Transforms source code, translating and inlining `formatMessage` calls
 **/

var Extractor = (function (_Visitor) {
  function Extractor(options) {
    _classCallCheck(this, Extractor);

    _Visitor.call(this, options);
    this.patterns = null;
  }

  _inherits(Extractor, _Visitor);

  Extractor.prototype.extract = function extract(_ref) {
    var sourceCode = _ref.sourceCode;
    var sourceFileName = _ref.sourceFileName;

    this.patterns = {};
    this.run({ sourceCode: sourceCode, sourceFileName: sourceFileName });
    return this.patterns;
  };

  Extractor.prototype.visitFormatCall = function visitFormatCall(path, traverser) {
    traverser.traverse(path); // pre-travserse children
    if (!this.isReplaceable(path)) {
      return;
    }var pattern = this.getStringValue(path.node.arguments[0]);
    var error = this.getPatternError(pattern);
    if (error) {
      this.reportError(path, "SyntaxError: pattern is invalid", error.message);
    } else {
      var key = this.getKey(pattern);
      this.patterns[key] = getKeyNormalized(pattern);
    }
  };

  Extractor.extract = function extract(source, options) {
    return new Extractor(options).extract(source);
  };

  Extractor.extractFromFiles = function extractFromFiles(files, options) {
    var extractor = new Extractor(options);
    var translations = {};
    extractor.forEachFile(files, function (source) {
      var patterns = extractor.extract(source);
      _core.Object.assign(translations, patterns);
    });
    var json = JSON.stringify((function () {
      var _JSON$stringify = {};
      _JSON$stringify[extractor.locale] = translations;
      return _JSON$stringify;
    })(), null, "  ");
    if (options.outFile) {
      extractor.emitFile(options.outFile, json);
    } else {
      extractor.emitResult(json);
    }
  };

  return Extractor;
})(Visitor);

module.exports = Extractor;