"use strict";

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

module.exports = formatMessage;
/*global Intl:false */

var MessageFormat = _interopRequire(require("message-format"));

var formats = MessageFormat.data.formats;
var number = formats.number;
var date = formats.date;
var time = formats.time;
var cache = formats.cache;

var enableCache = true;
var currentLocale = "en";
var currentTranslate = function (pattern) {
  return pattern;
};

function cacheable(key, fn) {
  if (!enableCache) {
    return fn();
  }if (!(key in cache)) cache[key] = fn();
  return cache[key];
}

function formatMessage(pattern, args, locale) {
  locale = locale || currentLocale;
  return cacheable(locale + ":format:" + pattern, function () {
    return new MessageFormat(currentTranslate(pattern, locale), locale, { cache: enableCache }).format;
  })(args);
}

formatMessage.translate = function (pattern, locale) {
  locale = locale || currentLocale;
  return currentTranslate(pattern, locale);
};

formatMessage.setup = function () {
  var _ref = arguments[0] === undefined ? {} : arguments[0];

  var cache = _ref.cache;
  var locale = _ref.locale;
  var translate = _ref.translate;

  if (typeof cache === "boolean") enableCache = cache;
  if (locale) currentLocale = locale;
  if (translate) currentTranslate = translate;
};

formatMessage.number = function (locale, value) {
  var style = arguments[2] === undefined ? "medium" : arguments[2];

  return cacheable(locale + ":number:" + style, function () {
    return typeof Intl !== "undefined" && Intl.NumberFormat ? new Intl.NumberFormat(locale, number[style]).format : function (arg) {
      return Number(arg).toLocaleString(locale, number[style]);
    };
  })(value);
};

formatMessage.date = function (locale, value) {
  var style = arguments[2] === undefined ? "medium" : arguments[2];

  return cacheable(locale + ":date:" + style, function () {
    return typeof Intl !== "undefined" && Intl.DateTimeFormat ? new Intl.DateTimeFormat(locale, date[style]).format : function (arg) {
      return new Date(arg).toLocaleDateString(locale, date[style]);
    };
  })(value);
};

formatMessage.time = function (locale, value) {
  var style = arguments[2] === undefined ? "medium" : arguments[2];

  return cacheable(locale + ":time:" + style, function () {
    return typeof Intl !== "undefined" && Intl.DateTimeFormat ? new Intl.DateTimeFormat(locale, time[style]).format : function (arg) {
      return new Date(arg).toLocaleTimeString(locale, time[style]);
    };
  })(value);
};