"use strict";

var _classCallCheck = require("babel-runtime/helpers/class-call-check")["default"];

var _inherits = require("babel-runtime/helpers/inherits")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

var Visitor = _interopRequire(require("./visitor"));

var _path = require("path");

var relative = _path.relative;
var resolve = _path.resolve;
var basename = _path.basename;
var pathJoin = _path.join;

var recast = _interopRequire(require("recast"));

var sourceMap = _interopRequire(require("source-map"));

var Parser = _interopRequire(require("message-format/parser"));

var Transpiler = _interopRequire(require("./transpiler"));

var builders = recast.types.builders;
var CallExpression = recast.types.namedTypes.CallExpression.toString();

/**
 * Transforms source code, translating and inlining `formatMessage` calls
 **/

var Inliner = (function (_Visitor) {
  function Inliner(options) {
    _classCallCheck(this, Inliner);

    _Visitor.call(this, options);
    this.declarations = [];
  }

  _inherits(Inliner, _Visitor);

  Inliner.prototype.inline = function inline(_ref) {
    var sourceCode = _ref.sourceCode;
    var sourceFileName = _ref.sourceFileName;
    var sourceMapName = _ref.sourceMapName;
    var inputSourceMap = _ref.inputSourceMap;

    this.declarations.length = 0;
    var ast = this.run({ sourceCode: sourceCode, sourceFileName: sourceFileName });
    if (!ast) {
      return { code: sourceCode, map: inputSourceMap };
    }ast.program.body = ast.program.body.concat(this.getDeclarations());

    sourceMapName = sourceMapName || sourceFileName + ".map";
    return this.print({ ast: ast, sourceMapName: sourceMapName, inputSourceMap: inputSourceMap });
  };

  Inliner.prototype.visitFormatCall = function visitFormatCall(path, traverser, opts) {
    traverser.traverse(path); // pre-travserse children
    if (!this.isReplaceable(path)) {
      return;
    }var node = path.node;
    var isTranslateOnly = opts.isTranslateOnly;

    var localeArg = node.arguments[isTranslateOnly ? 1 : 2];
    var locale = localeArg && this.getStringValue(localeArg) || this.locale;
    var functionName = this.functionName;
    var originalPattern = this.getStringValue(node.arguments[0]);
    var pattern = this.translate(originalPattern, locale);
    if (pattern == null) {
      pattern = this.handleMissingTranslation(originalPattern, locale, path);
    }
    if (isTranslateOnly) {
      return path.replace(builders.literal(pattern));
    }
    var paramsNode = node.arguments[1] || builders.literal(null);
    var patternAst = Parser.parse(pattern);

    var _Transpiler$transpile = Transpiler.transpile(patternAst, { locale: locale, functionName: functionName, paramsNode: paramsNode, originalPattern: originalPattern });

    var replacement = _Transpiler$transpile.replacement;
    var declaration = _Transpiler$transpile.declaration;

    this.addDeclaration(declaration);
    // body[0] should be an ExpressionStatement, so its expresion is what we want
    var codeAst = recast.parse(replacement).program.body[0].expression;
    if (codeAst.type === CallExpression) {
      codeAst.arguments[1] = paramsNode;
    }
    return path.replace(codeAst);
  };

  Inliner.prototype.handleMissingTranslation = function handleMissingTranslation(originalPattern, locale, path) {
    var replacement = this.missingReplacement || originalPattern;
    var message = "no " + locale + " translation found for key " + JSON.stringify(this.getKey(originalPattern));

    if (this.missingTranslation === "ignore") {} else if (this.missingTranslation === "warning") {
      this.reportWarning(path, "Warning: " + message);
    } else {
      // 'error'
      this.reportError(path, "Error: " + message);
      throw new Error(message);
    }

    return replacement;
  };

  Inliner.prototype.addDeclaration = function addDeclaration(decl) {
    if (decl && this.declarations.indexOf(decl) === -1) {
      this.declarations.push(decl);
    }
  };

  Inliner.prototype.getDeclarations = function getDeclarations() {
    var codeString = this.declarations.join("\n");
    var codeAst = recast.parse(codeString);
    this.declarations.length = 0;
    return codeAst.program.body;
  };

  Inliner.prototype.sourceMapComment = function sourceMapComment(path) {
    return "\n//# sourceMappingURL=" + path;
  };

  Inliner.prototype.sourceMapInlineComment = function sourceMapInlineComment(map) {
    return this.sourceMapComment("data:application/json;base64," + new Buffer(JSON.stringify(map)).toString("base64"));
  };

  Inliner.inline = function inline(source, options) {
    return new Inliner(options).inline(source);
  };

  Inliner.inlineFiles = function inlineFiles(files, options) {
    if (options.outDir) {
      Inliner.inlineManyToMany(files, options);
    } else {
      Inliner.inlineManyToOne(files, options);
    }
  };

  Inliner.inlineManyToMany = function inlineManyToMany(files, options) {
    var inliner = new Inliner(options);
    var root = resolve(options.root);
    var outDir = options.outDir.replace(/\/$/, "") + "/";
    inliner.forEachFile(files, function (source) {
      var result = inliner.inline(source);
      var outFileName = pathJoin(outDir, relative(root, source.sourceFileName));

      if (options.sourceMaps === "inline") {
        result.code += inliner.sourceMapInlineComment(result.map);
      } else if (options.sourceMaps) {
        var mapFileName = outFileName + ".map";
        inliner.emitFile(mapFileName, JSON.stringify(result.map));
        result.code += inliner.sourceMapComment(basename(mapFileName));
      }

      inliner.emitFile(outFileName, result.code, result.map);
    });
  };

  Inliner.inlineManyToOne = function inlineManyToOne(files, options) {
    var inliner = new Inliner(options);
    var outFileName = options.outFile || options.filename;
    var map = new sourceMap.SourceMapGenerator({
      file: outFileName
    });
    var code = "";
    var offset = 0;

    inliner.forEachFile(files, function (source) {
      var result = inliner.inline(source);
      var filename = source.souceFileName;
      var consumer = new sourceMap.SourceMapConsumer(result.map);
      map._sources.add(filename);
      map.setSourceContent(filename, source.sourceCode);
      consumer.eachMapping(function (mapping) {
        map._mappings.add({
          generatedLine: mapping.generatedLine + offset,
          generatedColumn: mapping.generatedColumn,
          originalLine: mapping.originalLine,
          originalColumn: mapping.originalColumn,
          source: filename
        });
      });

      code += result.code + "\n";
      offset = code.split("\n").length;
    });

    if (options.sourceMaps === "inline") {
      code += inliner.sourceMapInlineComment(map);
    } else if (options.sourceMaps) {
      var mapFileName = outFileName + ".map";
      inliner.emitFile(mapFileName, JSON.stringify(map));
      code += inliner.sourceMapComment(basename(mapFileName));
    }

    if (options.outFile) {
      inliner.emitFile(options.outFile, code, map);
    } else {
      inliner.emitResult(code);
    }
  };

  return Inliner;
})(Visitor);

module.exports = Inliner;

// do nothing