"use strict";

var _core = require("babel-runtime/core-js")["default"];

var _interopRequire = require("babel-runtime/helpers/interop-require")["default"];

exports.getTranslate = getTranslate;
exports.getGetKey = getGetKey;
exports.getKeyNormalized = getKeyNormalized;
exports.getKeyUnderscored = getKeyUnderscored;
exports.getKeyUnderscoredCrc32 = getKeyUnderscoredCrc32;
exports.__esModule = true;

var slug = _interopRequire(require("speakingurl"));

var crc32 = _interopRequire(require("crc32"));

var Parser = _interopRequire(require("message-format/parser"));

var Printer = _interopRequire(require("message-format/printer"));

function getTranslate(options) {
  if (options.translate) {
    return options.translate;
  }

  if (!options.translations) {
    return function (pattern) {
      return pattern;
    };
  }

  var getKey = getGetKey(options);
  return function (pattern, locales) {
    var translations = options.translations;
    var key = getKey(pattern);
    locales = [].concat(locales);
    for (var _iterator = locales, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _core.$for.getIterator(_iterator);;) {
      var _ref;

      if (_isArray) {
        if (_i >= _iterator.length) break;
        _ref = _iterator[_i++];
      } else {
        _i = _iterator.next();
        if (_i.done) break;
        _ref = _i.value;
      }

      var locale = _ref;

      var parts = locale.split("-");
      while (parts.length) {
        var current = parts.join("-");
        if (current in translations && key in translations[current]) {
          return translations[current][key];
        }
        parts.pop();
      }
    }
  };
}

function getGetKey(options) {
  switch (options.keyType) {
    case "underscored_crc32":
      return getKeyUnderscoredCrc32;
    case "underscored":
      return getKeyUnderscored;
    case "normalized":
      return getKeyNormalized;
    default:
      // 'literal'
      return function (pattern) {
        return pattern;
      };
  }
}

function getKeyNormalized(pattern) {
  return Printer.print(Parser.parse(pattern)).replace(/\s+/g, " ");
}

function getKeyUnderscored(pattern) {
  pattern = getKeyNormalized(pattern);
  return slug(pattern, { separator: "_", lang: false }).replace(/[-_]+/g, "_").slice(0, 50);
}

function getKeyUnderscoredCrc32(pattern) {
  pattern = getKeyNormalized(pattern);
  var underscored = slug(pattern, { separator: "_", lang: false }).replace(/[-_]+/g, "_").slice(0, 50);
  var crc = crc32(pattern.length + ":" + pattern).toString(16);
  return underscored + "_" + crc;
}